/** <?php /**
 * @name PHPIDS
 * @uses http://php-ids.org/
 *
 * @author Stefanie Janine Stoelting, mail@stefanie-stoelting.de
 * @modification Thomas Jakobi, thomas.jakobi@partout.info
 * @changes Use MODx logging/Use MODx email settings/use MODx Plugin configuration (don't need to patch config.ini and/or plugin code)
 * @package PHPIDS
 * @license LGPL
 * @since 2009/08/29
 * @version 0.6.1.1b
 *
 * <strong>0.6.1.1a</strong> A plugin to include PHPIDS into MODx to log and prevent intrusions
 * First download PHPDS from its homepage and load it up to your webspace.
 * If you install PHPIDS into a public path, you should remove webaccess to the
 * lib path, for example with the attached .htaccess file.
 *
 * This plugin is based on an article published by Heise Verlag:
 * Getting started with the PHPIDS intrusion detection system
 * http://www.h-online.com/security/Getting-started-with-the-PHPIDS-intrusion-detection-system--/features/113163
 *
 * Requirements: MODx 0.9.6 or newer, PHP 5.1.2 or newer, PHPIDS 0.6.1.1
 *
 * On page Systemevents check the entry OnWebPageInit
 *
 * For database logging please create the table as defined IDS/Log/Database.php
 *
 * For a test just call your site with the following parameters: index.php?id=1&test=">XXX
 *
 *
 * 
 * Plugin Configuration
 *
 * &useMODxBasePath=Use MODx base path?;list;true,false;true &basePath=PHPIDS path?;string;assets/lib/phpids/lib &useLogFile=Use log file?;list;true,false;false &$useEmail=Use email?;list;true,false;true &useDatabase=Use database?;list;true,false;true&useMODXLog=Use MODx Log?;list;true,false;true &logFromImpact=Start logging from impact?;int;5 &sendMailFromImpact=Start sending mails from impact, high (around 15) or very high (around 25-50) impact?;int;15 &idsRecipients=Mail resipients (separated by comma);text; &idsSubject=Mail subject (keep empty to use default);text;
 *
 **/

$useMODxBasePath    = isset($useMODxBasePath)? $useMODxBasePath=='true': true;
$useLogFile         = isset($useLogFile)? $useLogFile=='true': false;
$useEmail           = isset($useEmail)? $useEmail=='true': true;
$useDatabase        = isset($useDatabase)? $useDatabase=='true': true;
$useMODXLog         = isset($useMODXLog)? $useMODXLog=='true': true;
$logFromImpact      = isset($logFromImpact)? $logFromImpact: 5;
$sendMailFromImpact = isset($sendMailFromImpact)? $sendMailFromImpact: 25;
$idsRecipients      = isset($idsRecipients)? $idsRecipients : $modx->config['emailsender'];
$idsSubject         = isset($idsSubject)? $idsSubject : $modx->config['site_name'] . ': PHPIDS Plugin detected an intrusion attempt!';

if ($useMODXBasePath) {
  $basePath           = isset($basePath)? $modx->config['base_path']. 'assets/lib/phpids/lib': '';
} else {
  $basePath           = isset($basePath)? $basePath: '';
}

if (empty($basePath)) {

  $modx->logEvent(0, 2, printf( 'PHPIDS Plugin: $basePath not set!'));
  // Exeption is only shown, on backend login
  if (isset($_SESSION['mgrValidated'])? $_SESSION['mgrValidated']: false == 1) {
    $modx->messageQuit ('PHPIDS Plugin: $basePath not set!');
  }
  
} else {

  set_include_path(
     get_include_path()
     . PATH_SEPARATOR
     . $basePath
    );


  require_once 'IDS/Init.php';
  try {
    $request = array(
      'REQUEST' => $_REQUEST,
      'GET' => $_GET,
      'POST' => $_POST,
      'COOKIE' => $_COOKIE
    );

    $init = IDS_Init::init($basePath . '/IDS/Config/Config.ini');

    $init->config['General']['base_path'] = $basePath . '//IDS/';
    $init->config['General']['use_base_path'] = true;
    $init->config['Logging']['recipients'] = $idsRecipients;
    $init->config['Logging']['subject'] = $idsSubject;
    $init->config['Logging']['header'] = 'From: ' . $modx->config['emailsender'];
    $init->config['Caching']['caching'] = 'none';
    $init->config['Caching']['caching'] = 'none';

    $ids = new IDS_Monitor($request, $init);
    $result = $ids->run();
    $impact = $result->getImpact();

    if (!$result->isEmpty()) {

      require_once 'IDS/Log/Composite.php';

      $compositeLog = new IDS_Log_Composite();

      if ($useLogFile && $impact >= $logFromImpact) {

        require_once 'IDS/Log/File.php';
        $compositeLog->addLogger(IDS_Log_File::getInstance($init));

      }


      if ($useEmail && $impact >= $sendMailFromImpact) {

        require_once 'IDS/Log/Email.php';
        $compositeLog->addLogger(IDS_Log_Email::getInstance($init));

      }

      if ($useDatabase && $impact >= $logFromImpact) {

        require_once 'IDS/Log/Database.php';
        $compositeLog->addLogger(IDS_Log_Database::getInstance($init));

      }

      if ($useMODXLog && $impact >= $logFromImpact) {
          $modx->logEvent(0, 2, $result, 'PHPIDS Intrusion Alert! Impact: ' . $impact);
      }

      $compositeLog->execute($result);
    }

  } catch (Exception $e) {

    $modx->logEvent(0, 2, printf( 'An error occured: %s', $e->getMessage()), 'PHPIDS Plugin');
    // Exeption is only shown, on backend login
    if (isset($_SESSION['mgrValidated'])? $_SESSION['mgrValidated']: false == 1) {
      $modx->messageQuit ( sprintf ( 'An error occured: %s', $e->getMessage() ) );
    }

  }
}